import Base from 'ember-simple-auth/authenticators/base';
import fetch from 'fetch';
import RSVP from 'rsvp';
import { run } from '@ember/runloop';
import classic from 'ember-classic-decorator';

const headers = {
  accept: 'application/json',
  'content-type': 'application/json',
};

@classic
export default class TokenAuthenticator extends Base {
  hostUrl; // hook up on extended authenticator with app config hostUrl
  path = '/users';
  identificationAttributeName = 'email';
  serverTokenEndpoint = '/sign_in';
  invalidateEndpoint = '/sign_out';
  validateTokenEndpoint = '/validate_token';
  sessionType = 'Admin'; // Change this on Tenant app
  wrongAccountError =
    'Wrong account type. Please login with a valid account type.';

  authenticate = (identification, password, otp_attempt) => {
    return new RSVP.Promise(async (resolve, reject) => {
      const url = this._makeUrl(this.path, this.serverTokenEndpoint);

      const requestPayload = {
        [this.identificationAttributeName]: identification,
        password,
        otp_attempt,
      };

      try {
        const response = await fetch(url, {
          method: 'POST',
          body: JSON.stringify(requestPayload),
          headers,
        });

        const data = await response.json();

        if (!response.ok) return run(null, reject, data);
        if (data?.data?.role_type !== this.sessionType)
          run(null, reject, { errors: [this.wrongAccountError] });

        const authResults = this._getAuthSessionData(response);
        const requiresOtp = !authResults.accessToken; // If no authentication token is present in headers, we still need otp and data should include what type

        run(null, requiresOtp ? reject : resolve, {
          ...authResults,
          ...data.data,
        });
      } catch (error) {
        run(null, reject, error);
      }
    });
  };

  invalidate = (data) => {
    return new RSVP.Promise(async (resolve, reject) => {
      const url = this._makeUrl(this.path, this.invalidateEndpoint);
      try {
        const response = await fetch(url, {
          method: 'DELETE',
          headers: {
            ...headers,
            ...this._authHeaders(data),
          },
        });
        if (!response.ok) return run(null, reject, response);
        run(null, resolve, data);
      } catch (error) {
        run(null, reject, error);
      }
    });
  };

  restore = (data) => {
    return new RSVP.Promise(async (resolve, reject) => {
      const url = this._makeUrl(this.path, this.validateTokenEndpoint);
      try {
        const response = await fetch(url, {
          method: 'GET',
          headers: {
            ...headers,
            ...this._authHeaders(data),
          },
        });

        const newData = await response.json();
        if (!response.ok) return run(null, reject, newData);

        // Note: some auth data like token and jwt are rotated on revalidation so it important that
        //       newData and authResults come second so that they overwrite the cached data
        // Also Note: accessing data in headers here causes ember-simple-auth to redirect to login and then dashboard
        // so validateTokenEndpoint returns info in the body
        run(null, resolve, {
          ...data,
          ...newData.data,
        });
      } catch (error) {
        run(null, reject, error);
      }
    });
  };

  // Joins url string segments and ensures no double slashes
  _makeUrl = (...segments) => {
    if (!this.hostUrl) {
      console.error("Authenticator Error: Update token endpoint hostUrl");
      return;
    }
    const urlSegments = [this.hostUrl, ...segments];
    return urlSegments
      .map((segment) => segment.replace(/^\/|\/$/g, ''))
      .join('/');
  };

  _authHeaders = (sessionData) => {
    return {
      'access-token': sessionData.accessToken,
      client: sessionData.client,
      uid: sessionData.uid,
    };
  };

  _getAuthSessionData(response) {
    return {
      accessToken: response.headers.map['access-token'],
      expiry: response.headers.map['expiry'],
      tokenType: response.headers.map['token-type'],
      uid: response.headers.map['uid'],
      client: response.headers.map['client'],
    };
  }
}
